﻿using System;
using System.Collections.ObjectModel;
using System.Data.Services.Client;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using SilverlightAstoriaQA.NWServiceReference;

namespace SilverlightAstoriaQA
{
    public partial class Page : UserControl
    {
        private ObservableCollection<Product> _products = new ObservableCollection<Product>();
        private Entities _ctx = new Entities(new Uri("NWDataService.svc", UriKind.Relative));
        private ObservableCollection<OrderDetail> _orderDetails = new ObservableCollection<OrderDetail>();

        public Page()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(Page_Loaded);
            loadButton.Click += new RoutedEventHandler(LoadButton_Clicked);
            saveButton.Click += new RoutedEventHandler(SaveButton_Clicked);
            productDataGrid.SelectionChanged += new SelectionChangedEventHandler(productDataGrid_SelectionChanged);
        }

        void productDataGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Product product = productDataGrid.SelectedItem as Product;
            if (product == null) return;

            _orderDetails.Clear();

            if (product.OrderDetails == null || product.OrderDetails.Count == 0)
            {
                _ctx.BeginLoadProperty(product, "OrderDetails", FindOrderDetail_Completed, null);
            }
            else
            {
                LoadOrderDetails();
            }
        }

        private void LoadOrderDetails()
        {
            Product product = productDataGrid.SelectedItem as Product;
            if (product == null) return;

            var query = (from od in product.OrderDetails
                         orderby od.OrderID ascending
                         select od);
            foreach (OrderDetail item in query)
            {
                item.PropertyChanged += ((sender, e) =>
                {
                    OrderDetail entity = (OrderDetail)sender;
                    _ctx.UpdateObject(entity);
                });
                _orderDetails.Add(item);
            }
        }

        private void FindOrderDetail_Completed(IAsyncResult result)
        {
            _ctx.EndLoadProperty(result);
            Deployment.Current.Dispatcher.BeginInvoke(() => LoadOrderDetails());
        }

        private void SaveButton_Clicked(object sender, RoutedEventArgs e)
        {
            _ctx.BeginSaveChanges(SaveChangesOptions.Batch, new AsyncCallback(Save_Complete), null);
        }

        private void Save_Complete(IAsyncResult result)
        {
            bool succeeded = true;
            try
            {
                DataServiceResponse response = (DataServiceResponse)_ctx.EndSaveChanges(result);

                foreach (OperationResponse opResponse in response)
                {
                    if (opResponse.Error != null)
                    {
                        Debug.WriteLine(string.Concat("Failed to save: ", opResponse.StatusCode));
                        succeeded = false;
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(string.Concat("Save Operation Threw an Exception: ", ex));
                succeeded = false;
            }

            if (succeeded)
                MessageBox.Show("Saved new socks!");
            else
                MessageBox.Show("Failed to save new socks!");
        }

        private void LoadButton_Clicked(object sender, RoutedEventArgs e)
        {
            _ctx.MergeOption = MergeOption.OverwriteChanges; // makes sure we get fresh data and overwrite any changes (cancels local changes).
            FindProducts();
        }

        void Page_Loaded(object sender, RoutedEventArgs e)
        {
            productDataGrid.DataContext = _products;
            orderDetailsDataGrid.DataContext = _orderDetails;
        }

        private void FindProducts()
        {
            _products.Clear();
            DataServiceQuery<Product> dq = (
                from p in _ctx.ProductSet
                select p)
                as DataServiceQuery<Product>;
            dq.BeginExecute(new AsyncCallback(FindProduct_Completed), dq);


            return;
            DataServiceQuery<Product> dq1 = (
                from p in _ctx.ProductSet
                select p)
                as DataServiceQuery<Product>;


            DataServiceQuery<Product> dq3 = (
                from p in _ctx.ProductSet.Expand("OrderDetails/Orders")
                select p)
                as DataServiceQuery<Product>;

        
        }

        private void FindProduct_Completed(IAsyncResult result)
        {
            DataServiceQuery<Product> query = (DataServiceQuery<Product>)result.AsyncState;
            try
            {
                var entities = query.EndExecute(result);
                foreach (Product item in entities)
                {
                    item.PropertyChanged += ((sender, e) =>
                                                 {
                                                     Product entity = (Product)sender;
                                                     _ctx.UpdateObject(entity);
                                                 });
                    _products.Add(item);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Failed to retrieve data: " + ex.ToString());
            }
        }

        //protected void entity_PropertyChanged(object sender, PropertyChangedEventArgs e)
        //{
        //    Product entity = (Product)sender;
        //    _ctx.UpdateObject(entity);
        //}

        //Deployment.Current.Dispatcher.BeginInvoke(() => _products.Add(item));

    }
}
